package org.zoikks.drools.xlsx;

import java.io.InputStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import junit.framework.TestCase;

import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.definition.KnowledgePackage;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.junit.Test;
import org.zoikks.drools.ProcessApp;
import org.zoikks.drools.facts.AbstractFinding;
import org.zoikks.drools.facts.FVC;
import org.zoikks.drools.facts.IVC;
import org.zoikks.drools.facts.Patient;
import org.zoikks.drools.facts.SVC;
import org.zoikks.drools.facts.VC;

public class SampleWorkflowXlsx extends TestCase {

    private Workbook wb;
    private KnowledgeBase kbase;

    public enum ParamType {
        AGE, DLCO, FEV1, FEV1_PBd, FVC, FVC_PBd, GENDER, HEIGHT, IVC, RACE, SVC, TLC_MULTI, TLC_PLETH, TLC_SINGLE, VC, HGB, CoHGB
    };

    public SampleWorkflowXlsx() {

        this.configureProcesses();
        this.configureXlsxFile();
    }

    @Test
    public void testSampleCheckSpecificRowFromXlsxSheet() {

        Sheet sheet = this.wb.getSheetAt(0);
        Sheet resultSheet = this.wb.getSheetAt(1);

        int rowNum = 6;

        Row row = sheet.getRow(rowNum);
        Row resultRow = resultSheet.getRow(rowNum);

        this.check(row, resultRow);
    }

    @Test
    public void testSampleXlsxSheets() {

        Sheet sheet = this.wb.getSheetAt(0);
        Sheet resultSheet = this.wb.getSheetAt(1);

        for (Row row : sheet) {

            // Need to skip the first row of header info.
            if (row.getRowNum() == 0) {
                continue;
            }

            Row resultRow = resultSheet.getRow(row.getRowNum());

            this.check(row, resultRow);
        }
    }

    private void check(Row row, Row resultRow) {

        StatefulKnowledgeSession ksession = this.kbase.newStatefulKnowledgeSession();

        long empi = this.grabValue(row.getCell(0)).longValue();

        System.out.println("Testing patient #: " + empi);

        Map<String, Object> params = new HashMap<String, Object>();
        params.put("fev1", this.createParameter(this.grabValue(row.getCell(5)), ParamType.FEV1));
        params.put("fev1-pbd", this.createParameter(this.grabValue(row.getCell(6)), ParamType.FEV1_PBd));
        params.put("fvc", this.createParameter(this.grabValue(row.getCell(7)), ParamType.FVC));
        params.put("fvc-pbd", this.createParameter(this.grabValue(row.getCell(8)), ParamType.FVC_PBd));

        Patient patient = this.createPatient(this.grabValue(row.getCell(1)), this.grabValue(row.getCell(2)),
                this.grabValue(row.getCell(3)), this.grabValue(row.getCell(4)));

        ksession.insert(patient);

        this.insertParamsAsFacts(params, ksession);

        ksession.startProcess("pft", params);
        ksession.fireAllRules();

        ksession.dispose();
        ksession = null;
    }

    private void insertParamsAsFacts(Map<String, Object> parameters, StatefulKnowledgeSession ksession) {

        Set<String> keys = parameters.keySet();

        for (String key : keys) {

            if (parameters.get(key) != null) {
                ksession.insert(parameters.get(key));
            }
        }
    }

    private Patient createPatient(Double race, Double gender, Double age, Double height) {

        Patient patient = new Patient(gender.longValue(), height.doubleValue(), age.doubleValue(), race.longValue());

        return patient;
    }

    private Double grabValue(Cell cell) {

        if (cell != null) {
            if (cell.getCellType() == Cell.CELL_TYPE_NUMERIC) {
                double value = cell.getNumericCellValue();
                return value;
            }
        }

        return null;
    }

    private AbstractFinding createParameter(Double value, ParamType paramType) {

        if (value == null) {
            return null;
        }

        if (paramType == ParamType.FVC) {
            return new FVC(value, null);
        }

        if (paramType == ParamType.FVC_PBd) {

            FVC fvc = new FVC(value, null);
            fvc.setPostBD(true);

            return fvc;
        }

        if (paramType == ParamType.IVC) {
            return new IVC(value, null);
        }

        if (paramType == ParamType.SVC) {
            return new SVC(value, null);
        }

        if (paramType == ParamType.VC) {
            return new VC(value, null);
        }

        return null;
    }

    private void configureProcesses() {

        final KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();

        kbuilder.add(ResourceFactory.newClassPathResource("vc-grading.drl", ProcessApp.class), ResourceType.DRL);
        kbuilder.add(ResourceFactory.newClassPathResource("vc-lln.drl", ProcessApp.class), ResourceType.DRL);
        kbuilder.add(ResourceFactory.newClassPathResource("vc-max.drl", ProcessApp.class), ResourceType.DRL);
        kbuilder.add(ResourceFactory.newClassPathResource("vc-predicted.drl", ProcessApp.class), ResourceType.DRL);

        kbuilder.add(ResourceFactory.newClassPathResource("processes/pft.bpmn", ProcessApp.class), ResourceType.BPMN2);
        kbuilder.add(ResourceFactory.newClassPathResource("processes/vc-max-calculations.bpmn", ProcessApp.class),
                ResourceType.BPMN2);

        if (kbuilder.hasErrors()) {
            System.out.println(kbuilder.getErrors().toString());
            throw new RuntimeException("Unable to compile.");
        }

        final Collection<KnowledgePackage> pkgs = kbuilder.getKnowledgePackages();

        this.kbase = KnowledgeBaseFactory.newKnowledgeBase();
        this.kbase.addKnowledgePackages(pkgs);
    }

    private void configureXlsxFile() {

        try {
            InputStream inp = (this.getClass().getClassLoader().getResourceAsStream("PFT-TestValues.xlsm"));

            this.wb = WorkbookFactory.create(inp);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

}